<?php

namespace SGalinski\SgMail\Updates;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use SGalinski\SgMail\Service\BackendService;
use TYPO3\CMS\Install\Updates\AbstractUpdate;

/**
 * Migrate template db entries to the correct root pages
 */
class MigrateData extends AbstractUpdate {
	/**
	 * @var string
	 */
	protected $title = 'Find all templates & queue entries without site root and assign the correct site root id';

	/**
	 * @var \TYPO3\CMS\Extbase\Object\ObjectManager
	 */
	protected $objectManager = NULL;

	/**
	 * @var array
	 */
	protected $tables = [
		'tx_sgmail_domain_model_mail', 'tx_sgmail_domain_model_template'
	];

	/**
	 * Checks whether updates are required.
	 *
	 * @param string &$description The description for the update
	 * @return bool Whether an update is required (TRUE) or not (FALSE)
	 */
	public function checkForUpdate(&$description) {
		$description = 'Set the site_root_id for all the queue entries and templates if not set or pid = 0';

		$databaseConnection = $this->getDatabaseConnection();
		$where = 'site_root_id = 0';

		foreach ($this->tables as $table) {
			$result = $databaseConnection->exec_SELECTquery('*', $table, $where);
			if ($result->num_rows > 0) {
				return TRUE;
			}
		}

		return FALSE;
	}

	/**
	 * Performs the accordant updates.
	 *
	 * @param array &$dbQueries Queries done in this update
	 * @param mixed &$customMessages Custom messages
	 * @return bool Whether everything went smoothly or not
	 */
	public function performUpdate(array &$dbQueries, &$customMessages) {
		$databaseConnection = $this->getDatabaseConnection();

		foreach ($this->tables as $table) {
			$where = 'site_root_id = 0';
			$result = $databaseConnection->exec_SELECTquery('uid, pid', $table, $where)->fetch_all();
			$dbQueries[] = $databaseConnection->debug_lastBuiltQuery;

			/** @var array $result */
			foreach ($result as $row) {
				$siteRootId = BackendService::getSiteRoot($row[1]);

				if ($siteRootId === NULL) {
					$this->retrieveFirstSiteRoot();
				}

				$where = 'uid = ' . $row[0];
				$databaseConnection->exec_UPDATEquery($table, $where, ['site_root_id' => $siteRootId]);
				$dbQueries[] = $databaseConnection->debug_lastBuiltQuery;
			}
		}

		$this->markWizardAsDone();
		return TRUE;
	}

	/**
	 * get the first site root that is not pid = 0
	 *
	 * @return int
	 */
	private function retrieveFirstSiteRoot() {
		$databaseConnection = $this->getDatabaseConnection();
		$where = "is_siteroot = 1 AND uid <> '0'";
		$pagesResult = $databaseConnection->exec_SELECTgetSingleRow('uid', 'pages', $where, '', '', '1');
		$dbQueries[] = $databaseConnection->debug_lastBuiltQuery;

		if (empty($pagesResult)) {
			return 0;
		} else {
			return $pagesResult[0][0];
		}
	}
}
